#!/usr/bin/env python

"""Description
Setup script for BINOCH: Binding Inferrence from Nucleosome Occupancy Changes.
Copyright (c) 2010 Cliff Meyer <cliff@jimmy.harvard.edu>

This code is free software; you can redistribute it and/or modify it
under the terms of the Artistic License (see the file COPYING included
with the distribution).

@status:  beta
@version: $Revision$
@author:  Cliff Meyer
@contact: cliff@jimmy.harvard.edu
"""

import os
import sys
#from distutils.core import setup, Extension
from setuptools import setup, Extension

def main():
    if not float(sys.version[:3])>=2.4:
        sys.stderr.write("CRITICAL: Python version must be greater than or equal to 2.4! python 2.6.1 is recommended!\n")
        sys.exit(1)
 
    try:
        import numpy
        NUMPY_PATH = numpy.__path__[0]
        print NUMPY_PATH
    except ImportError, e:
        sys.stderr.write("CRITICAL: numpy 1.3 or greater must be installed\n")
        sys.exit(1)

    setup(name="BINOCH",
        version="1.0.0",
        description="BINOCH: Binding Inferrence from Nucleosome Occupancy Changes\n. Sequence tools for motif analysis from ChIP-seq and nucleosome occupancy data.",
        author='Cliff Meyer',
        author_email='cliff@jimmy.harvard.edu',
        url='http://liulab.dfci.harvard.edu/',
        package_dir={'':'src'},
        packages=['cistrome','cistrome.test','seq','seq.test','motifIO','motifIO.test','motifanalysis','motifanalysis.test','table','table.test'],
        package_data={'cistrome':['data/*.conf','test/data/test.bed','test/data/test2.bed'],'motifIO':['PWM/*.xml'],'motifanalysis':['test/data/*.xls','test/data/*.bed'],'table':['test/data/*.bed']},
        ext_package='seq',
        ext_modules = [
            Extension( '_seq',
            sources = ['src/seq/seq.c','src/seq/genomescan_Markov0_3.c'],
            #include_dirs=['seq','/usr/local/lib/python2.5/site-packages/numpy/core/include']
            include_dirs=['seq',NUMPY_PATH+'/core/include']
            )
        ],
        scripts=['bin/binoch','bin/tagtab'],
        #test_suite='cistrome.test.test',
        classifiers=[
            'Development Status :: 5 - productive',
            'Environment :: Console',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: Artistic License',
            'Operating System :: MacOS :: MacOS X',
            'Programming Language :: Python',
            ],
        )

if __name__ == '__main__':
    main()
