import sys,time,os,math,struct,random,numpy
import __init__
#import seqpos.seqpos as seqpos
#import re,MySQLdb,operator 
#import mystats

NONEARESTSITE = 999999999
MISSINGVAL    = -99999999
NOCLUSTER     = -1
ERRCODE  = 1
PASSCODE = 0

def maskseq( seq, masklocs ):
    """
    seq is a list of sequence strings
    masklocs is a list of 3 tuples / lists: 
        x[0] index of string to be masked
        x[1] start position to be masked
        x[2] end position + 1 
    """
    for xloc in masklocs:
        l = list( seq[xloc[0]] )
        l[xloc[1]:xloc[2]] = (xloc[2]-xloc[1])*['N']
        seq[xloc[0]] = ''.join(l)
    return


class interval(object):

    def __init__(self,genome,description=''):
        self.description = description
        self.genome = genome
        self.chrom  = None 
        self.start  = None
        self.end    = None
        self.idx    = None
        self.length = None
        self.prob   = None
        self.val    = None
        self.name   = None
        self.namelookup = None # MakeIndex makes a dictionary of names mapping to indices 
        self.mid    = None
        self.strand = None
        self.seq    = None
        self.conservation = None
        self.motifscores  = None
        self.basecontent  = None
        self.info   = None # list of strings holding information about ChIP 

    def __str__(self):
        """
        print bed file
        """
        s=''
        for i,elem in enumerate(self.chrom):
            s += '\t'.join([self.chrom[i],str(self.start[i]),str(self.end[i])])
            if self.name and (len(self.name) == len(self.chrom)):
                s+= '\t%s' % self.name[i]
            if self.val != None: # and len(self.val) == len(self.chrom):
                s+= '\t%5.2f' % self.val[i]
            if self.strand != None: # and len(self.strand) == len(self.chrom):
                s+= '\t%s' % self.strand[i]
           
            s += '\n'
        return s


    def read_bed(self,file,scorefield=4):
        """
        Scorefield is the column to be read as score (index starting at 0).
        """

        CHROMFIELD,STARTFIELD,ENDFIELD,NAMEFIELD = range(4)
        SCOREFIELD = scorefield
        STRANDFIELD = 5

        try:
            fp=open(file,'r')
        except IOError:
            print "%s not found." % file
            raise

        self.chrom, self.start, self.end, self.strand = [],[],[],None
        self.length, self.mid = [],[]
    
        fi = iter(fp)

        nameflag,scoreflag,strandflag = False,False,False
        for i,line in enumerate(fi):
            if line[0:3] == 'chr':
 
                try:
                    field    = line.split()
                    chrom_t  = field[CHROMFIELD]
                    start_t  = int(field[STARTFIELD])
                    end_t    = int(field[ENDFIELD])
                    length_t = end_t-start_t
                    mid_t    = int(0.5*(end_t+start_t))
                except:
                    chrom_t,start_t,end_t,length_t,mid_t = None,None,None,None,None
                    print "Error reading line of bedfile %s on line %d:\n%s" % (file,i,line)
                    continue
                    #return ERRCODE

                if chrom_t:
                    self.chrom  += [chrom_t]
                    self.start  += [start_t]
                    self.end    += [end_t]
                    self.length += [length_t]
                    self.mid    += [mid_t]

                    if len(self.chrom) == 1:
                        if len(field) > NAMEFIELD:
                            nameflag = True
                            self.name = []
                        if len(field) > SCOREFIELD:
                            scoreflag = True
                            self.val = []
                        if len(field) > STRANDFIELD and field[STRANDFIELD] in ['+','-']:
                            strandflag = True
                            self.strand = []

                    if nameflag == True:
                        try:
                            self.name   += [field[NAMEFIELD]]
                        except IndexError:
                            print "Error reading name in bedfile %s on line %d:\n%s" % (file,i,line)
                            return ERRCODE
                    
                    if scoreflag == True:
                        try:
                            self.val    += [float(field[SCOREFIELD])]
                        except:
                            print "Error reading score in bedfile %s on line %d:\n%s" % (file,i,line)
                            return ERRCODE

                    if strandflag == True:
                        try:
                            self.strand += [field[STRANDFIELD]]
                            if self.strand[-1] not in ['-','+']:
                                print "Error reading strand in bedfile %s on line %d:\n%s" % (file,i,line)
                                print "Bed file error: strand must be + or - %s" % strand[-1]
                                return ERRCODE
                        except:
                            print "Error reading bedfile %s on line %d:\n%s" % (file,i,line)
                            return ERRCODE

                        #self.strand += [field[STRANDFIELD]]

        #print "number of regions:", len(self.chrom)
        return PASSCODE


    def rank( self, Field='prob', order='decreasing' ):
        """
        Rank regions by Field
        Rank is placed in 'idx' Field
        order can be 'increasing' or 'decreasing'
        """
        if order == 'increasing':
            t = sorted( [(x,i) for i,x in enumerate(self.__dict__[Field])] )
        else: 
            t = sorted( [(-x,i) for i,x in enumerate(self.__dict__[Field])] )

        self.idx = len(self.chrom)*[0]
        for i,x in enumerate(t):
            self.idx[x[1]] = i 


    def MakeIndex( self, Field="name" ):
        """
        returns dictionary mapping identifiers in Field to index
        """
        d = {}
        for i,elem in enumerate( self.__dict__[Field] ):
            d[elem] = i
        return d


    def copy( self, new=None ):
        """
        Copy interval object
        """

        if new == None:
            new = interval(self.genome,description=self.description)

        for elem in self.__dict__:
            # copy lists 
            if type( self.__dict__[elem] ) == type([]):
                new.__dict__[elem] = self.__dict__[elem]
            # copy arrays
            if type( self.__dict__[elem] ) == type(numpy.zeros(1,float)):
                new.__dict__[elem] = self.__dict__[elem]
            # copy dictionaries
            if type( self.__dict__[elem] ) == type({}):
                new.__dict__[elem] = self.__dict__[elem]

        return new


    def Select( self, Field='length', UpperBound=None, LowerBound=None, SelectNames=None, SelectionType='standard' ):
        """
        This function can be used to define a new ChIP_region object which is a subset of the original ChIP regions where the subset is 
        defined by bound criteria on ChIP region attributes.
        Any attribute with data can be used, for example: 'chrom', 'length', etc
        Returns object with regions that fulfill criteria of ChIP.Field < UpperBound and ChIP.Field >= LowerBound
        If SelectionType is set to 'complement' regions that fulfill the criteria of ChIP.Field > UpperBound or ChIP.Field < LowerBound are returned.
        A string field can be selected by specifying the lower bound only.
        """

        # for string comparisons UpperBound is set to be next string after LowerBound
        if type( self.__dict__[Field] ) == type( [] ) and len( self.__dict__[Field] ) > 0:
            if type( LowerBound ) == type( "0" ) and ( UpperBound == None or UpperBound == LowerBound ):
                UpperBound = LowerBound + chr(0)

        if SelectNames != None:

            if SelectionType == 'standard':
                idxmap = self.MakeIndex( Field )
                idx = [ idxmap[x] for i,x in enumerate(SelectNames) if ( x in idxmap )]
            elif SelectionType == 'complement':
                idxmap = {}
                for i,elem in enumerate(SelectNames):
                    idxmap[elem] = i 
                idx = [ i for i,x in enumerate(self.__dict__[Field]) if ( x not in idxmap )]
            else:
                return None
            idx = sorted(idx)
 
        else:

            if SelectionType == 'standard':
                if LowerBound != None and UpperBound != None:
                    idx = [ i for i,x in enumerate(self.__dict__[Field]) if ( x!=None and x>=LowerBound and x<UpperBound )]
                elif LowerBound != None:
                    idx = [ i for i,x in enumerate(self.__dict__[Field]) if ( x!=None and x>=LowerBound )]
                elif UpperBound != None:
                    idx = [ i for i,x in enumerate(self.__dict__[Field]) if ( x!=None and  x<UpperBound )]
                else:
                    idx = [ i for i,x in enumerate(self.__dict__[Field]) if ( x!=None )]
            elif SelectionType == 'complement':
                if LowerBound != None and UpperBound != None:
                    idx = [ i for i,x in enumerate(self.__dict__[Field]) if ( x!=None and ( x<LowerBound or x>=UpperBound )) ]
                elif LowerBound != None:
                    idx = [ i for i,x in enumerate(self.__dict__[Field]) if ( x!=None and x<LowerBound ) ]
                elif UpperBound != None:
                    idx = [ i for i,x in enumerate(self.__dict__[Field]) if ( x!=None and x>=UpperBound ) ]
                else:
                    idx = [ i for i,x in enumerate(self.__dict__[Field]) if ( x!=None )]
            else:
                return None


        if idx == None or len( idx ) == 0:
            return None

        new = interval(self.genome)
        #new.genome = self.genome
        for elem in self.__dict__:
            # copy lists 
            if type( self.__dict__[elem] ) == type([]):
                try:
                    new.__dict__[elem] = [ self.__dict__[elem][i] for i in idx ]
                except IndexError:
                    print "Error in regions. Select trying to copy field %s" % elem
                    raise IndexError
            # copy arrays
            if type( self.__dict__[elem] ) == type(numpy.zeros(1,numpy.float)):
                try:
                    new.__dict__[elem] = self.__dict__[elem][idx]
                except IndexError:
                    print "Error in regions.Select trying to copy field %s" % elem
                    raise IndexError
 
            # copy dictionaries
            if type( self.__dict__[elem] ) == type({}):
                try:
                    new.__dict__[elem] = self.__dict__[elem]
                except IndexError:
                    print "Error in regions.Select trying to copy field %s" % elem
                    raise IndexError
 
        if SelectionType == 'standard':
            new.description = self.description + "_select_" + Field + "_" + ">" + str(LowerBound) + "& <" + str(UpperBound)
        else:   
            new.description = self.description + "_select_" + Field + "_" + "<" + str(LowerBound) + "| >" + str(UpperBound)

        return new



    def getPhast(self):
        """
        download phastCons scores for all regions in a bed file.
        """
        genome = self.genome
        build_dict = __init__.UCSCPHASTCONSPATH
        #build_dict = {"hg18":UCSC_HG18_PHASTCONS, "mm9":UCSC_MM9_PHASTCONS}
        try:
            build_dir = build_dict[genome]
        except KeyError:
            print "Path to PhastCons files not found."
            raise

        self.conservation = []
        for i,chr in enumerate(self.chrom):
            name = chr + ".dat"
            file = open( os.path.join(build_dir, name ),'rb')
            file.seek(0,2)
            fileend = file.tell()
            skip = (self.start[i]-1) * 4
            #print name,skip
            if skip < fileend - 4:
                file.seek(skip,0)
                self.conservation.append([])
                for x in range( self.start[i], self.end[i] ):
                    score = struct.unpack("f",file.read(4))[0]
                    if score < 0:
                        score = None
                    self.conservation[i].append(score)
            else:
                self.conservation.append( ( self.end[i] - self.start[i] )*[0.0] )

            file.close()


    def getSequence(self, repeatMasked=False):
        """
        Retrieve DNA sequence corresponding to each bed region. 
        getSequence( genome, repeatMasked )
        repeatMasked defaults to False. 
        """
 
        masked = __init__.UCSCMASKEDSEQPATH
        nomask = __init__.UCSCNOMASKSEQPATH

        if repeatMasked == True:
            build_dict = masked 
        else:
            build_dict = nomask
 
        genome = self.genome
        build_dir = build_dict[genome]
        self.seq = []

        for i,x in enumerate( self.chrom ):

            chr   = x
            left  = self.start[i]
            right = self.end[i]

            if repeatMasked == True:
                try:
                    chr_file = os.path.join( build_dir,  chr + ".fa.masked" )
                    fpchr = open( chr_file, 'r' )
                except:
                    print "Please check your that this path is correctly specified in your datapath.conf file: %s." % (build_dir)
                    print "and that this file is there %s." % (chr_file)
                    self.seq.append( None )
                    continue

            else:

                try:
                    subdir = os.listdir( build_dir )
                    chr_file = os.path.join( build_dir, chr + ".fa" )
                    fpchr = open( chr_file, 'r' )
                except:
                    print "Could not open file:", chr_file
                    self.seq.append( None )
                    raise IOError

            fpchr.readline()
            here = fpchr.tell()
            line = fpchr.readline()
            line = line.rstrip()
            line_len = len(line)

            # shift to 0 offset
            left  -= 1
            right -= 1

            start = ( left  / line_len ) * ( line_len + 1 ) + ( left  % line_len )  
            stop  = ( right / line_len ) * ( line_len + 1 ) + ( right % line_len )  

            try:
                fpchr.seek( start + here )
            except:
                print 'ERROR: invalid chr location', start, here, left, right, line_len, x
                sys.exit()
                
            seq = fpchr.read( stop - start )
            seq = seq.replace( '\n', '' )
 
            self.seq.append( seq )
                
            fpchr.close()

    def checkseq(self):
        basedict = {'A':0,'C':1,'G':2,'T':3,'N':4}
        for i,seqelem in enumerate(self.seq):
            seqelem = list( seqelem.upper() )
            if len( [ elem for elem in seqelem if elem not in basedict ] ) > 0:
                print "unknown base in sequence %d %s %d %d %s" % ( i, self.chrom[i], self.start[i], self.end[i], self.seq[i] )
        return

    def CountBases(self):
        self.basecontent = [] # ACGT fraction
        for i,seqelem in enumerate(self.seq):
            seqelem = list( seqelem.upper() )
            n = float(len(seqelem))
            self.basecontent.append( [ sum( [ 1.0 for x in seqelem if x == base ] )/n for base in ['A','C','G','T'] ] )
            self.basecontent[-1] += [ sum( [ 1.0 for i,x in enumerate(seqelem[0:-1]) if seqelem[i:i+2] == ['C','G'] ] )/n ]
        return


    def TrimBed(self, length=600):
        """
        Make all bed regions of length 'length' keeping the midpoint the same ie defined as "mid" ( which could be defined as other than midway between start and end ).
        """
        for i,x in enumerate( self.start ):
            self.start[i] = int( max( 1, self.mid[i] - length/2.0 ) )
            self.end[i] = int( self.mid[i] + length/2.0 )
            self.length[i] = self.start[i] - self.end[i]
            self.seq = None
            self.conservation = None


    def PadBed(self, pad=300, side='both'):
        """
        pad each bed region ie add pad bases to each end of the bed region
        side: 'left', 'right', or 'both'
        """
        for i,x in enumerate(self.start):
            start_t,end_t = self.start[i],self.end[i]
            if side == 'left' or side == 'both':
                self.start[i] = min( max( 1, self.start[i] - pad ), self.end[i] )
            if side == 'right' or side == 'both':
                self.end[i] = max( self.start[i], self.end[i] + pad )
            self.length[i] = self.end[i] - self.start[i]
            if self.length[i] < 1:
                print "region too short %s\t%d\t%d" % (self.chrom[i],start_t,end_t)
                raise ValueError, "%d" % self.length[i]  
        self.seq = None
        self.conservation = None
        return


    def CpG_ratio(self):
        """
        count the number of C and G s in each sequence
        compute the ratio of CpG to expected CpG
        """
        ratio = []
        CorG  = []
        for i,telem in enumerate(self.seq):
            elem = telem.upper()
            n = float(len(elem))
            CorG.append( ( elem.count('C') + elem.count('G') )/n )
            t = elem.count('CG')
            if CorG[-1] > 0:
                ratio.append( (t/n)/( 0.25*CorG[-1]*CorG[-1] ) ) 
            else:
                ratio.append( 0 ) 
        return CorG,ratio


    def order(self):
        """
        Use: self.order()
        sorts entries in to genomic order by chromosone  chr1, chr2, and position.
        """

        _Chrom_,_Start_,_End_,_Index_ = range(4)
        SelfZList = zip( self.chrom, self.start, self.end, range( len(self.chrom) ) )
        SelfZList.sort()
        idx = [ elem[_Index_] for elem in SelfZList ]

        # copy region object 
        new = interval(self.genome)
        new.description = self.description + "_ordered"

        for elem in self.__dict__:
            # copy lists 
            if type( self.__dict__[elem] ) == type([]):
                try:
                    new.__dict__[elem] = [ self.__dict__[elem][i] for i in idx ]
                except IndexError:
                    print "Error in regions. order trying to copy field %s" % elem
                    raise IndexError
            # copy arrays
            if type( self.__dict__[elem] ) == type(numpy.zeros(1,numpy.float)):
                try:
                    new.__dict__[elem] = self.__dict__[elem][idx]
                except IndexError:
                    print "Error in regions. order trying to copy field %s" % elem
                    raise IndexError
            # copy dictionaries
            if type( self.__dict__[elem] ) == type({}):
                try:
                    new.__dict__[elem] = self.__dict__[elem]
                except IndexError:
                    print "Error in regions. order trying to copy field %s" % elem
                    raise IndexError

        return new


    def distance( self, other, nsites = 4, distance_type  = 'abs'):
        """
        Use: self.distance( other, nsites,  distance_type ).
        For every region in self find the distance to the nearest n regions in other.
        distance_type: 'abs' absolute distance between centers or 'oriented' distance upstream or downstream of 
        self midpoint where orientation is determined by self.strand.
        """

        SELF  = 0
        OTHER = 1
        _Chrom_, _Mid_, _Strand_, _Name_, _SelfOther_ = 0, 1, 2, 3, 4

        if self.mid is None or len(self.mid) != len(self.chrom):
            raise ValueError, "Distance requires midpoint data."
        if self.strand is None or len(self.strand) != len(self.chrom):
            raise ValueError, "Distance requires strand data."
        if self.name is None or len(self.name) != len(self.chrom):
            raise ValueError, "Distance requires name data."
 
        if other.mid is None or len(other.mid) != len(other.chrom):
            raise ValueError, "Distance requires midpoint data."
        if other.name is None or len(other.name) != len(other.chrom):
            raise ValueError, "Distance requires name data."

        SelfZList  = zip( self.chrom,  self.mid,  self.strand,  self.name,  len(self.chrom)*[SELF]   )
        OtherZList = zip( other.chrom, other.mid, len(other.chrom)*['+'], other.name, len(other.chrom)*[OTHER] )

        for i,elem in enumerate( SelfZList ):
            if '' in elem:
                print "error in self list distance function", elem 
                sys.exit()

        for i,elem in enumerate( OtherZList ):
            if '' in elem:
                print "error in other list distance function", elem 
                sys.exit()

        locA  = SelfZList
        locB  = OtherZList
        listB = [] 
        dictA = {}
        selfidxlookup = {}
        self.nearest_site      = [] 
        self.nearest_site_name = [] 
 
        for i,x in enumerate(self.name):
            self.nearest_site.append(None)
            self.nearest_site_name.append(None)
            selfidxlookup[x] = i

        mergelist = locA + locB
        mergelist.sort()
        orientation = { '+':1, '-':-1 }
 
        for x in mergelist:
            if not x[ _SelfOther_ ] == SELF:
                listB.append( x )
            else:
                dictA[ x[ _Name_ ] ] = [ [y[ _Name_ ], orientation[ x[ _Strand_ ] ]*orientation[ y[ _Strand_ ]]*int( y[ _Mid_ ] - x[ _Mid_ ] )] for y in listB[ -nsites:len(listB)] if y[ _Chrom_ ] == x[ _Chrom_ ] ]

        mergelist.reverse()
        listB = []

        for x in mergelist:
            if not x[ _SelfOther_ ] == SELF:
                listB.append( x )
            else:
                dictA[ x[ _Name_ ] ] += [[ y[ _Name_ ], orientation[ x[ _Strand_ ] ]*orientation[ y[ _Strand_ ] ]*int( y[ _Mid_ ] - x[ _Mid_ ])] for y in listB[ -nsites:len(listB)] if y[ _Chrom_ ] == x[ _Chrom_ ] ]

        for elem in dictA:
            dictA[elem].sort( lambda x,y: cmp( abs(x[1]), abs(y[1]) ) )

            self.nearest_site_name[selfidxlookup[elem]] = [ x[0] for x in dictA[elem][0:nsites] ]
            if distance_type == "abs": 
                self.nearest_site[selfidxlookup[elem]] = [ abs(x[1]) for x in dictA[elem][0:nsites] ] 
            else:
                self.nearest_site[selfidxlookup[elem]] = [ x[1] for x in dictA[elem][0:nsites] ] 

        return dictA
 

