import os,unittest
import numpy
import seq.count
import seq._seq

class TestSeqscan(unittest.TestCase):
    def testscore(self):

        seqs = \
        ["ATATATATATACGCGCGCGCGCGCGACACACACACACACACACACACAC",\
        "ATATATATATACTCGCGCGCGCGCGACACACACACACACACACACACAC",\
        "ATATATATATACTGGCGCGCGCGCGACACACACACACACACACACACAC",\
        "ACTGGGGTACTGAGGCGCGCGCGCGACACACACACACACACACACACAC",\
        "ATATATATATACGCGCGCGCGCGCGACACACAGTACACACACCCCCAGT",\
        "ACTGATATACTGATACTGCGCGCGCGCGCGACACACACACACACACACC",\
        "ACTGATATACTGATACTGCGCGCGCGCGCGACTGACACACACACACACC"]

        start_t    = [26,10,10,0,45,0,0]
        end_t      = [30,14,14,4,49,4,4]
        strand_t   = [1,0,0,0,1,0,0]

        prob = seq.count.count(seqs)
        option = seq._seq.MAX_OPTION
        order  = 2
        pssm = numpy.array( [ [1,0,0,0], [0,1,0,0], [0,0,0,1], [0,0,1,0] ] ,numpy.float )
        idxmax,startmax,endmax,orientmax,scoremax = seq._seq.seqscan( seqs, pssm, prob, order, option )
        print '--------------------'
        for i,elem in enumerate(idxmax):
            print elem,startmax[i],endmax[i],orientmax[i],scoremax[i],seqs[elem][startmax[i]:endmax[i]] 
            self.assertEqual(i,elem)
            self.assertEqual( startmax[i],  start_t[i],  "motif position incorrect")
            self.assertEqual( orientmax[i], strand_t[i], "strand incorrect")
            if i > 0:
                self.assertTrue(scoremax[i]>=scoremax[i-1])

        start_t = [-1,-1,-1,-1,-1,-1,-1]
        end_t   = [-1,-1,-1,-1,-1,-1,-1]

        option = seq._seq.MEAN_OPTION
        pssm = numpy.array( [[1,0,0,0],[0,1,0,0],[0,0,0,1],[0,0,1,0]], numpy.float )
        idxmean,startmean,endmean,orientmean,scoremean = seq._seq.seqscan( seqs, pssm, prob, order, option )
        print '--------------------'
        for i,elem in enumerate(idxmax):
            print elem,startmean[i],endmean[i],orientmean[i],scoremean[i],seqs[elem][startmean[i]:endmean[i]] 
            self.assertEqual(i,elem)
            self.assertEqual(startmean[i],start_t[i],"motif position incorrect")
            if i > 0:
                self.assertTrue(scoremean[i]>=scoremean[i-1])





def suite():
    suite = unittest.TestSuite()
    suite.addTest(TestSeqscan('testscore'))
    return suite

if __name__ == "__main__":
    unittest.main()
