import motifxml
import motifxml.base
import os,os.path,unittest
import motifanalysis
import motifanalysis.nsd
import random
import numpy,math
import seq.count as cnt
import seq._seq
import cistrome.regions as cis

def revcomp( l ):
    """
    Generate the reverse complement of a sequence.
    """
    comp = { 'A':'T', 'C':'G', 'G':'C', 'T':'A', 'N':'N' }
    l.reverse()
    r = [comp[x] for x in l]
    return r

def simulation(pssm,n=1000,w=200,mu0=0,sd0=1,mu1=1,sd1=1,f=0.05,sdpos=1,seed=1):
    """
    simulate DNA sequence and NSD scores
    pssm: motif matrix
    n: number of sequences to generare
    w: length of each sequence
    mu0: mean of score associated with unbound sequence
    sd0: sd of score associated with unbound sequence
    mu1: mean of score associated with bound sequence
    sd1: sd of score associated with bound sequence
    f: fraction of bound sequences
    sdpos: sd of binding around center of sequence
    """

    random.seed()

    base = ['A','C','G','T']
    motifseq = []
    for elem in pssm:
        row = []
        for i in range(4):
            row += elem[i]*[base[i]]
        motifseq.append(row)

    seql   = []
    scorel = []
    bindl  = []
    # generate random sequence 
    # for f percent of sequences generate a dna sequence from pssm and  score from N(mu1,sd1)
    mlen = len(motifseq)

    posl = []

    for i in range(n):
        seq  = [random.sample(base,1)[0] for i in range(w)]
        if random.random() < f:
            motif = [random.sample( motifseq[k],1)[0] for k in range(mlen) ]
            if random.random() < 0.5:
                motif = revcomp(motif)
            #pos = random.randint(0,w-5)
            pos = int(w*random.gauss(0.5,sdpos))
            if pos > 0 and pos < w-mlen-1:
                #pos = max(0,pos)
                #pos = min(w-mlen-1,pos)
                #print pos/float(w)
                posl += [pos+mlen/2.0]
                seq[pos:pos+mlen] = motif
                #print 'ttt',posl[-1],motif
            #print motif
            scorel += [random.gauss(mu1,sd1)]
            bindl += [1]
        else:
            scorel += [random.gauss(mu0,sd0)]
            bindl += [0]
        seql += [''.join(seq)]

    return seql,numpy.array(scorel),bindl

class TestNSD(unittest.TestCase):

    def test_NSD_EM(self):
        field = ["dist"]
        low = [2000]
        up  = [10000000]
        phastcon = 0

        nuc = ["veh_nuc","DHT_3h_nuc"]
        lin = ["veh_lin","DHT_3h_lin"]
        genome = "hg18"

        known_motifs = motifxml.base.MotifList()
        known_motifs_file_name = os.path.join(motifxml.PWMPATH,"transfac.xml")
        known_motifs.from_xml_file(known_motifs_file_name)
        test = ['M00447']
 
        datafile = os.path.join( os.path.split( __file__ )[0], 'data/test.10k.xls' )
        out = motifanalysis.nsd.main( datafile, known_motifs, genome=genome, select_id=field, select_L=low, select_U=up, nuc_id=nuc, lin_id=lin, score_option='max', conservation_cutoff=phastcon,output_file='binoch_test_AR_EM.txt', bias=False, names=test, printnsd=False )
        print out["DHT_3h_nuc"][0]
        #self.assertEqual( out["DHT_3h_nuc"][0] )
        self.assertEqual(1,1)

if __name__ == "__main__":
    unittest.main()
